# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

class PulseConfiguration:

    def __init__(self, **kwargs):

        # Default values for Mozilla pulse
        defaults = {
            # Connection defaults
            'user':       'public',
            'password':   'public',
            'host':       'pulse.mozilla.org',
            'port':       5672,
            'vhost':      '/',
            # Message defaults
            'serializer': 'json',
            'broker_timezone': 'US/Pacific',
        }

        # Set any vaiables passed in
        for key in kwargs:
            setattr(self, key, kwargs[key])

        # Set defaults for anything that isn't passed in
        for key in defaults:
            try:
                tmp = getattr(self, key)
            except AttributeError:
                setattr(self, key, defaults[key])



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from base import *
from mozillapulse.utils import email_to_routing_key

# ------------------------------------------------------------------------------
# Generic base class for messages that have to do with bugs
# ------------------------------------------------------------------------------

class GenericBugMessage(GenericMessage):

    def __init__(self):
        super(GenericBugMessage, self).__init__()
        self.routing_parts.append('bug')

    def _required_data_fields(self):
        tmp = super(GenericBugMessage, self)._required_data_fields()
        tmp.append('who')
        tmp.append('when')
        return tmp

    def set_bugdata(self, bugdata):
        self.set_data('bug', bugdata)

# ------------------------------------------------------------------------------
# High-level bug state messages (for end-user convenience)
# ------------------------------------------------------------------------------

class BugStateMessage(GenericBugMessage):

    def __init__(self, what):
        super(BugStateMessage, self).__init__()
        self.routing_parts.append(what)

class BugCreatedMessage(BugStateMessage):

    def __init__(self):
        super(BugCreatedMessage, self).__init__('new')

class BugClosedMessage(BugStateMessage):

    def __init__(self):
        super(BugClosedMessage, self).__init__('closed')

class BugReopenedMessage(BugStateMessage):

    def __init__(self):
        super(BugReopenedMessage, self).__init__('reopened')


# TODO: Should this have top-level fields?
class BugUndupedMessage(BugStateMessage):

    def __init__(self):
        super(BugUndupedMessage, self).__init__('unduped')

class BugDupedMessage(BugStateMessage):

    def __init__(self):
        super(BugDupedMessage, self).__init__('duped')

    def _required_data_fields(self):
        tmp = super(BugDupedMessage, self)._required_data_fields()
        tmp.append('dupe')
        tmp.append('original')
        return tmp

# ------------------------------------------------------------------------------
# Messages about changing bug values
# ------------------------------------------------------------------------------

class BugAddedMessage(GenericBugMessage):

    def __init__(self, what):
        super(BugAddedMessage, self).__init__()
        self.routing_parts.append('added')
        self.routing_parts.append(what)

    def _required_data_fields(self):
        tmp = super(BugAddedMessage, self)._required_data_fields()
        tmp.append('bug')
        tmp.append('value')
        return tmp

class BugRemovedMessage(GenericBugMessage):

    def __init__(self, what):
        super(BugRemovedMessage, self).__init__()
        self.routing_parts.append('removed')
        self.routing_parts.append(what)

    def _required_data_fields(self):
        tmp = super(BugRemovedMessage, self)._required_data_fields()
        tmp.append('bug')
        tmp.append('value')
        return tmp

class BugChangedMessage(GenericBugMessage):
    
    def __init__(self, what):
        super(BugChangedMessage, self).__init__()
        self.routing_parts.append('changed')
        self.routing_parts.append(what)

    def _required_data_fields(self):
        tmp = super(BugChangedMessage, self)._required_data_fields()
        tmp.append('bug')
        return tmp

    def _validate(self):
        super(BugChangedMessage, self)._validate()
        if 'before' not in self.data and 'after' not in self.data:
            raise MalformedMessage('require either "before" or "after" field')

        if not self.data['before'] and not self.data['after']:
            raise MalformedMessage('before and after fields cannot both be blank')

        if self.data['before'] == self.data['after']:
            raise MalformedMessage('before and after fields cannot be the same')

# ------------------------------------------------------------------------------
# Messages about requests
# ------------------------------------------------------------------------------

class BugRequestMessage(BugChangedMessage):

    def __init__(self, what, type, who=None):
        super(BugRequestMessage, self).__init__(what)
        self.routing_parts.append(type)
        if who:
            self.routing_parts.append(email_to_routing_key(who))

class BugRequestAddedMessage(BugRequestMessage):

    def __init__(self, what, who=None):
        super(BugRequestAddedMessage, self).__init__(what, 'requested', who)

class BugRequestGrantedMessage(BugRequestMessage):

    def __init__(self, what, who=None):
        super(BugRequestGrantedMessage, self).__init__(what, 'granted', who)

class BugRequestDeniedMessage(BugRequestMessage):

    def __init__(self, what, who=None):
        super(BugRequestDeniedMessage, self).__init__(what, 'denied', who)

class BugRequestCanceledMessage(BugRequestMessage):

    def __init__(self, what, who=None):
        super(BugRequestCanceledMessage, self).__init__(what, 'canceled', who)



#!/usr/bin/env python
#
# Copyright (c) 2009, 2010, Henry Precheur <henry@precheur.org>
#
# Permission to use, copy, modify, and/or distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
# REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
# INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
# LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
# OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
# PERFORMANCE OF THIS SOFTWARE.
#
'''Formats dates according to the :RFC:`3339`.'''

__author__ = 'Henry Precheur <henry@precheur.org>'
__license__ = 'ISCL'
__version__ = '3'
__all__ = ('rfc3339', )

import datetime
import time
import unittest

def _timezone(utcoffset):
    '''
    Return a string representing the timezone offset.

    >>> _timezone(0)
    '+00:00'
    >>> _timezone(3600)
    '+01:00'
    >>> _timezone(-28800)
    '-08:00'
    >>> _timezone(-1800)
    '-00:30'
    '''
    # Python's division uses floor(), not round() like in other languages:
    #   -1 / 2 == -1 and not -1 / 2 == 0
    # That's why we use abs(utcoffset).
    hours = abs(utcoffset) // 3600
    minutes = abs(utcoffset) % 3600 // 60
    if utcoffset < 0:
        plusminus = '-'
    else:
        plusminus = '+'

    return '%c%02d:%02d' % (plusminus, hours, minutes)

def _timedelta_to_seconds(timedelta):
    '''
    >>> _timedelta_to_seconds(datetime.timedelta(hours=3))
    10800
    >>> _timedelta_to_seconds(datetime.timedelta(hours=3, minutes=15))
    11700
    '''
    return (timedelta.days * 86400 + timedelta.seconds +
            timedelta.microseconds // 1000)

def _utc_offset(date, use_system_timezone):
    '''
    Return the UTC offset of `date`. If `date` does not have any `tzinfo`, use
    the timezone informations stored locally on the system.

    >>> if time.localtime().tm_isdst:
    ...     system_timezone = -time.altzone
    ... else:
    ...     system_timezone = -time.timezone
    >>> _utc_offset(datetime.datetime.now(), True) == system_timezone
    True
    >>> _utc_offset(datetime.datetime.now(), False)
    0
    '''
    if isinstance(date, datetime.datetime) and date.tzinfo is not None:
        return _timedelta_to_seconds(date.dst() or date.utcoffset())
    elif use_system_timezone:
        t = time.mktime(date.timetuple())
        if time.localtime(t).tm_isdst: # pragma: no cover
            return -time.altzone
        else:
            return -time.timezone
    else:
        return 0

def _utc_string(d):
    return d.strftime('%Y-%m-%dT%H:%M:%SZ')

def rfc3339(date, utc=False, use_system_timezone=True):
    '''
    Return a string formatted according to the :RFC:`3339`. If called with
    `utc=True`, it normalizes `date` to the UTC date. If `date` does not have
    any timezone information, uses the local timezone::

        >>> d = datetime.datetime(2008, 4, 2, 20)
        >>> rfc3339(d, utc=True, use_system_timezone=False)
        '2008-04-02T20:00:00Z'
        >>> rfc3339(d) # doctest: +ELLIPSIS
        '2008-04-02T20:00:00...'

    If called with `user_system_time=False` don't use the local timezone if
    `date` does not have timezone informations and consider the offset to UTC
    to be zero::

        >>> rfc3339(d, use_system_timezone=False)
        '2008-04-02T20:00:00+00:00'

    `date` must be a `datetime.datetime`, `datetime.date` or a timestamp as
    returned by `time.time()`::

        >>> rfc3339(0, utc=True, use_system_timezone=False)
        '1970-01-01T00:00:00Z'
        >>> rfc3339(datetime.date(2008, 9, 6), utc=True,
        ...         use_system_timezone=False)
        '2008-09-06T00:00:00Z'
        >>> rfc3339(datetime.date(2008, 9, 6),
        ...         use_system_timezone=False)
        '2008-09-06T00:00:00+00:00'
        >>> rfc3339('foo bar')
        Traceback (most recent call last):
        ...
        TypeError: excepted datetime, got str instead
    '''
    # Check if `date` is a timestamp.
    try:
        if utc:
            return _utc_string(datetime.datetime.utcfromtimestamp(date))
        else:
            date = datetime.datetime.fromtimestamp(date)
    except TypeError:
        pass
    if isinstance(date, datetime.date):
        utcoffset = _utc_offset(date, use_system_timezone)
        if utc:
            if not isinstance(date, datetime.datetime):
                date = datetime.datetime(*date.timetuple()[:3])
            return _utc_string(date + datetime.timedelta(seconds=utcoffset))
        else:
            return date.strftime('%Y-%m-%dT%H:%M:%S') + _timezone(utcoffset)
    else:
        raise TypeError('excepted %s, got %s instead' %
                        (datetime.datetime.__name__, date.__class__.__name__))


class LocalTimeTestCase(unittest.TestCase):
    '''
    Test the use of the timezone saved locally. Since it is hard to test using
    doctest.
    '''

    def setUp(self):
        local_utcoffset = _utc_offset(datetime.datetime.now(), True)
        self.local_utcoffset = datetime.timedelta(seconds=local_utcoffset)
        self.local_timezone = _timezone(local_utcoffset)

    def test_datetime(self):
        d = datetime.datetime.now()
        self.assertEqual(rfc3339(d),
                         d.strftime('%Y-%m-%dT%H:%M:%S') + self.local_timezone)

    def test_datetime_timezone(self):

        class FixedNoDst(datetime.tzinfo):
            'A timezone info with fixed offset, not DST'

            def utcoffset(self, dt):
                return datetime.timedelta(hours=2, minutes=30)

            def dst(self, dt):
                return None

        fixed_no_dst = FixedNoDst()

        class Fixed(FixedNoDst):
            'A timezone info with DST'

            def dst(self, dt):
                return datetime.timedelta(hours=3, minutes=15)

        fixed = Fixed()

        d = datetime.datetime.now().replace(tzinfo=fixed_no_dst)
        timezone = _timezone(_timedelta_to_seconds(fixed_no_dst.\
                                                   utcoffset(None)))
        self.assertEqual(rfc3339(d),
                         d.strftime('%Y-%m-%dT%H:%M:%S') + timezone)

        d = datetime.datetime.now().replace(tzinfo=fixed)
        timezone = _timezone(_timedelta_to_seconds(fixed.dst(None)))
        self.assertEqual(rfc3339(d),
                         d.strftime('%Y-%m-%dT%H:%M:%S') + timezone)

    def test_datetime_utc(self):
        d = datetime.datetime.now()
        d_utc = d + self.local_utcoffset
        self.assertEqual(rfc3339(d, utc=True),
                         d_utc.strftime('%Y-%m-%dT%H:%M:%SZ'))

    def test_date(self):
        d = datetime.date.today()
        self.assertEqual(rfc3339(d),
                         d.strftime('%Y-%m-%dT%H:%M:%S') + self.local_timezone)

    def test_date_utc(self):
        d = datetime.date.today()
        # Convert `date` to `datetime`, since `date` ignores seconds and hours
        # in timedeltas:
        # >>> datetime.date(2008, 9, 7) + datetime.timedelta(hours=23)
        # datetime.date(2008, 9, 7)
        d_utc = datetime.datetime(*d.timetuple()[:3]) + self.local_utcoffset
        self.assertEqual(rfc3339(d, utc=True),
                         d_utc.strftime('%Y-%m-%dT%H:%M:%SZ'))

    def test_timestamp(self):
        d = time.time()
        self.assertEqual(rfc3339(d),
                         datetime.datetime.fromtimestamp(d).\
                         strftime('%Y-%m-%dT%H:%M:%S') + self.local_timezone)

    def test_timestamp_utc(self):
        d = time.time()
        d_utc = datetime.datetime.utcfromtimestamp(d) + self.local_utcoffset
        self.assertEqual(rfc3339(d),
                         (d_utc.strftime('%Y-%m-%dT%H:%M:%S') +
                          self.local_timezone))

    # If these tests start failing it probably means there was a policy change
    # for the Pacific time zone.
    # See http://en.wikipedia.org/wiki/Pacific_Time_Zone.
    if 'PST' in time.tzname:
        def testPDTChange(self):
            '''Test Daylight saving change'''
            # PDT switch happens at 2AM on March 14, 2010

            # 1:59AM PST
            self.assertEqual(rfc3339(datetime.datetime(2010, 3, 14, 1, 59)),
                             '2010-03-14T01:59:00-08:00')
            # 3AM PDT
            self.assertEqual(rfc3339(datetime.datetime(2010, 3, 14, 3, 0)),
                             '2010-03-14T03:00:00-07:00')

        def testPSTChange(self):
            '''Test Standard time change'''
            # PST switch happens at 2AM on November 6, 2010

            # 0:59AM PDT
            self.assertEqual(rfc3339(datetime.datetime(2010, 11, 7, 0, 59)),
                             '2010-11-07T00:59:00-07:00')

            # 1:00AM PST
            # There's no way to have 1:00AM PST without a proper tzinfo
            self.assertEqual(rfc3339(datetime.datetime(2010, 11, 7, 1, 0)),
                             '2010-11-07T01:00:00-07:00')


if __name__ == '__main__': # pragma: no cover
    import doctest
    doctest.testmod()
    unittest.main()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

# Custom exceptions
class InvalidRoutingKey(Exception):
    pass

class MalformedMessage(Exception):
    pass

# Base class that other objects should inherit from
class GenericMessage(object):
    
    def __init__(self, routing_key=None):
        self.routing_parts = []
        self.routing_key = routing_key
        self.data = {}
        self.metadata = {}

    # Setter for data items
    def set_data(self, key, value=None):
        self.data[key] = value

    # Subclasses can override to generate the key from stored data
    def _prepare_routing_key(self):
        self.routing_key = '.'.join(self.routing_parts)

    # Subclasses can override to define data fields that should be non-empty
    def _required_data_fields(self):
        return []

    # Check that required data fields are non-empty by default
    # Subclasses should override this with message-specific logic and then call
    # super to get this default validation
    def _validate(self):
        for field in self._required_data_fields():
            if field not in self.data or not self.data[field]:
                raise MalformedMessage('Missing required field: %s' % field)

    # Called right before the message is sent
    def _prepare(self):
        
        # Fill out the routing key (if needed)
        self._prepare_routing_key()
        
        # Check for a blank routing key
        if not self.routing_key:
            raise InvalidRoutingKey(self.routing_key)

        # Check for a routing key that has two periods in a row
        if self.routing_key.count('..'):
            raise InvalidRoutingKey(self.routing_key)

        # Make sure the message iteself is well-formed
        self._validate()

    # Pretty printing
    def __str__(self):
        return u'%s instance: %s: %s' % (self.__class__, self.routing_key, self.data)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from base import *

# ------------------------------------------------------------------------------
# Generic base class for messages that have to do with builds
# ------------------------------------------------------------------------------

# TODO: This isn't generic
class BuildMessage(GenericMessage):
    def __init__(self, event):
        super(BuildMessage, self).__init__()
        self.routing_parts.append(event['event'])
        self.metadata['master_name'] = event['master_name']
        self.metadata['master_incarnation'] = event['master_incarnation']
        self.metadata['message_id'] = event['id']

        for key, value in event['payload'].items():
            self.set_data(key, value)

# ------------------------------------------------------------------------------
# Base class for messages that have to do with posting builds to ftp
# ------------------------------------------------------------------------------

class BuildPostedMessage(GenericMessage):
    def __init__(self):
        super(BuildPostedMessage, self).__init__()
        self.routing_parts.append('build')
        self.routing_parts.append('posted')

    def _required_data_fields(self):
        tmp = super(BuildPostedMessage, self)._required_data_fields()
        tmp.append('build_id')
        tmp.append('revision')
        tmp.append('url')
        tmp.append('repository')
        tmp.append('product')
        tmp.append('product_version')
        tmp.append('locale')
        tmp.append('platform')
        tmp.append('build_date')
        tmp.append('package_type')
        return tmp

# ------------------------------------------------------------------------------
# Builds that are posted and meant to be downloaded and applied manually
# ------------------------------------------------------------------------------

class ManualBuildPostedMessage(BuildPostedMessage):
    def __init__(self):
        super(ManualBuildPostedMessage, self).__init__()
        self.routing_parts.append('manual')

# ------------------------------------------------------------------------------
# Builds that are posted and meant to be downloaded and applied as updates
# ------------------------------------------------------------------------------

class UpdateBuildPostedMessage(BuildPostedMessage):
    def __init__(self):
        super(UpdateBuildPostedMessage, self).__init__()
        self.routing_parts.append('update')

# ------------------------------------------------------------------------------
# Partial updates that are posted to ftp
# ------------------------------------------------------------------------------

class PartialUpdateBuildPostedMessage(UpdateBuildPostedMessage):
    def __init__(self):
        super(PartialUpdateBuildPostedMessage, self).__init__()
        self.routing_parts.append('partial')

    def _required_data_fields(self):
        tmp = super(PartialUpdateBuildPostedMessage, self)._required_data_fields()
        tmp.append('for_build')
        return tmp

# ------------------------------------------------------------------------------
# Full updates that are posted to ftp
# ------------------------------------------------------------------------------

class FullUpdateBuildPostedMessage(UpdateBuildPostedMessage):
    def __init__(self):
        super(FullUpdateBuildPostedMessage, self).__init__()
        self.routing_parts.append('full')



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from amqp import ChannelError
from kombu import Connection, Exchange, Queue

from config import PulseConfiguration
from utils import *

# Exceptions we can raise
class InvalidTopic(Exception):
    pass

class InvalidAppLabel(Exception):
    pass

class InvalidCallback(Exception):
    pass

class MalformedMessage(Exception):
    pass


class GenericConsumer(object):
    """Generic publisher class that specific consumers inherit from.
    FIXME: Mandatory properties, like "topic", should not be set from generic
    functions like configure() but should instead be explicitly required
    somewhere, e.g. the constructor.
    """

    def __init__(self, config, exchange=None, connect=True, heartbeat=False,
                 **kwargs):
        self.config     = config
        self.exchange   = exchange
        self.connection = None
        self.durable    = False
        self.applabel   = ''
        self.heartbeat  = heartbeat
        for x in ['applabel','topic','callback','durable']:
            if x in kwargs:
                setattr(self, x, kwargs[x])
                del kwargs[x]

        if connect:
            self.connect()

    def configure(self, **kwargs):
        """Sets variables."""

        for x in kwargs:
            setattr(self, x, kwargs[x])

    def connect(self):
        if not self.connection:
            self.connection = Connection(hostname=self.config.host,
                                         port=self.config.port,
                                         userid=self.config.user,
                                         password=self.config.password,
                                         virtual_host=self.config.vhost)

    def disconnect(self):
        if self.connection:
            self.connection.release()
            self.connection = None

    def purge_existing_messages(self):
        """Purge messages that are already in the queue on the broker.
        TODO: I think this is only supported by the amqp backend.
        """

        if self.durable and not self.applabel:
            raise InvalidAppLabel('Durable consumers must have an applabel')

        if not self.connection:
            self.connect()

        queue = self._create_queue(self.applabel)
        try:
            queue(self.connection).purge()
        except ChannelError, e:
            if e.message == 404:
                pass
            raise

    def queue_exists(self):
        self._check_params()
        if not self.connection:
            self.connect()

        queue = self._create_queue(self.applabel)
        try:
            queue(self.connection).queue_declare(passive=True)
        except ChannelError, e:
            if e.message == 404:
                return False
            raise
        return True

    def delete_queue(self):
        self._check_params()
        if not self.connection:
            self.connect()

        queue = self._create_queue(self.applabel)
        try:
            queue(self.connection).delete()
        except ChannelError, e:
            if e.message != 404:
                raise

    def listen(self, callback=None):
        """Blocks and calls the callback when a message comes into the queue.
        For info on one script listening to multiple channels, see
        http://ask.github.com/carrot/changelog.html#id1.
        """

        # One can optionally provide a callback to listen (if it wasn't already)
        if callback:
            self.callback = callback

        self._check_params()

        if not self.connection:
            self.connect()

        exchange = Exchange(self.exchange, type='topic')

        # Create a queue and bind to the first key.
        queue = self._create_queue(self.applabel, exchange, self.topic[0])
        consumer = self.connection.Consumer(queue, callbacks=[self.callback])

        # Bind to any additional keys.
        for routing_key in self.topic[1:]:
            consumer.queues[0].bind_to(exchange, routing_key)


        if self.heartbeat:
            hb_exchange = Exchange('org.mozilla.exchange.pulse.test',
                                   type='topic')
            consumer.queues[0].bind_to(hb_exchange, 'heartbeat')

        with consumer:
            while True:
                self.connection.drain_events()

        # Likely never get here but can't hurt.
        self.disconnect()

    def _check_params(self):
        if not self.exchange:
            raise InvalidExchange(self.exchange)

        if not self.topic:
            raise InvalidTopic(self.topic)

        if self.durable and not self.applabel:
            raise InvalidAppLabel('Durable consumers must have an applabel')

        if not self.callback or not hasattr(self.callback, '__call__'):
            raise InvalidCallback(self.callback)

        # We support multiple bindings if we were given an array for the topic.
        if not isinstance(self.topic, list):
            self.topic = [self.topic]

    def _create_queue(self, name, exchange=None, routing_key=''):
        return Queue(name=name,
                     exchange=exchange,
                     routing_key=routing_key,
                     durable=self.durable,
                     exclusive=False,
                     auto_delete=not self.durable)


# ------------------------------------------------------------------------------
# Consumers for various topics
# ------------------------------------------------------------------------------

class PulseTestConsumer(GenericConsumer):

    def __init__(self, **kwargs):
        super(PulseTestConsumer, self).__init__(PulseConfiguration(**kwargs), 'org.mozilla.exchange.pulse.test', **kwargs)


class PulseMetaConsumer(GenericConsumer):

    def __init__(self, **kwargs):
        super(PulseMetaConsumer, self).__init__(PulseConfiguration(**kwargs), 'org.mozilla.exchange.pulse', **kwargs)


class BugzillaConsumer(GenericConsumer):

    def __init__(self, **kwargs):
        super(BugzillaConsumer, self).__init__(PulseConfiguration(**kwargs), 'org.mozilla.exchange.bugzilla', **kwargs)


class CodeConsumer(GenericConsumer):

    def __init__(self, **kwargs):
        super(CodeConsumer, self).__init__(PulseConfiguration(**kwargs), 'org.mozilla.exchange.code', **kwargs)


class HgConsumer(CodeConsumer):

    def __init__(self, **kwargs):
        pass
        #super(CodeConsumer, self).__init__(PulseConfiguration(**kwargs), 'hg.push.mozilla.central', **kwargs)


class BuildConsumer(GenericConsumer):

    def __init__(self, **kwargs):
        super(BuildConsumer, self).__init__(PulseConfiguration(**kwargs), 'org.mozilla.exchange.build', **kwargs)


class NormalizedBuildConsumer(GenericConsumer):

    def __init__(self, **kwargs):
        super(NormalizedBuildConsumer, self).__init__(PulseConfiguration(**kwargs), 'org.mozilla.exchange.build.normalized', **kwargs)


class QAConsumer(GenericConsumer):

    def __init__(self, **kwargs):
        super(QAConsumer, self).__init__(PulseConfiguration(**kwargs), 'org.mozilla.exchange.qa', **kwargs)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

__all__ = ['config','utils','messages','rfc3339','consumers','publishers']

# Printing throws an error if we are printing using ascii
import sys
reload(sys)
sys.setdefaultencoding('utf-8')
